@@ -1,11 +1,15 @@
 #include <stdlib.h>
 #include <stddef.h>
 #include <ctype.h>
+#include <string.h>
+
+#ifdef EVHTP_HAS_SYS_TYPES
+#include <sys/types.h>
+#endif
 
 #include "internal.h"
 #include "evhtp/parser.h"
 #include "evhtp/config.h"
-#include "log.h"
 
 #if '\n' != '\x0a' || 'A' != 65
 #error "You have somehow found a non-ASCII host. We can't build here."
@@ -213,6 +217,7 @@ static const char * errstr_map[] = {
     "htparse_error_invalid_chunk",
     "htparse_error_invalid_state",
     "htparse_error_user",
+    "htparse_error_status",
     "htparse_error_unknown"
 };
 
@@ -2187,16 +2192,19 @@ hdrline_start:
                     const char * pe      = (const char *)(data + len);
                     size_t       to_read = _MIN_READ(pe - pp, p->content_len);
 
-                    if (to_read > 0)
-                    {
+                    if (to_read > 0) {
                         res = hook_body_run(p, hooks, pp, to_read);
 
                         i  += to_read - 1;
                         p->content_len -= to_read;
                     }
 
-                    if (p->content_len == 0)
-                    {
+                    if (res) {
+                        p->error = htparse_error_user;
+                        return i + 1;
+                    }
+
+                    if (p->content_len == 0) {
                         res      = hook_on_msg_complete_run(p, hooks);
                         p->state = s_start;
                     }
